import sys

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import connections
from django.db.backends.utils import CursorWrapper  # noqa

from .check__base import CheckBase, DbInfoMixin


class DbAvailableCheck(CheckBase, DbInfoMixin):
    """Check database availability."""

    message_template = "DB IS AVAILABLE [{}]"

    def check(self) -> bool:
        """Check method implementation."""
        try:
            for alias in settings.DATABASES:
                self.message = self.message_template.format(alias)
                self.extra_info = self._get_database_extra_info(alias)
                with connections[alias].cursor() as cursor:  # type: CursorWrapper
                    cursor.execute("SELECT 1")
                    self._print_check(self.message)
            return True
        except ImproperlyConfigured:
            self.stdout.write("DATABASES ARE NOT CONFIGURED!")
            sys.exit(1)
        except Exception as exc:
            self._print_check(self.message, exception=exc, exta_info=self.extra_info)
        return False



import sys
import time
from argparse import ArgumentParser
from collections import deque
from typing import TYPE_CHECKING, Any, Deque, Dict, Tuple, cast

from django.core.management.base import BaseCommand

from .waiter_checks.check_caches import CachesAvailableCheck
from .waiter_checks.check_celery import CeleryBrokerAvailableCheck
from .waiter_checks.check_db_available import DbAvailableCheck
from .waiter_checks.check_migrate import MigrateCheck

if TYPE_CHECKING:
    from .waiter_checks.check__base import CheckBase


class Command(BaseCommand):
    """Waiter for system checks before starting the main process.

    Checks the availability of project's services dependencies.
    """

    help = (  # noqa: A003
        "Waiter for system checks before starting the main process. "
        "Checks the availability of project's services dependencies."
    )

    def __init__(self) -> None:
        super().__init__()
        self._checks_queue: Deque[CheckBase] = deque()
        self._attempts = 6
        self._timeout = 10

    def add_arguments(self, parser: ArgumentParser) -> None:
        """Add arguments to command."""
        parser.add_argument(
            "-n",
            action="store",
            type=int,
            default=self._attempts,
            required=False,
            help=f"Attempts count (default {self._attempts})",
        )
        parser.add_argument(
            "-t",
            action="store",
            type=int,
            default=self._timeout,
            help=f"Timeout in seconds between attempts (default {self._timeout})",
            required=False,
        )

    def register_checks(self) -> None:
        """Register checks."""
        self._checks_queue.extend((
            DbAvailableCheck(self.stdout),
            # TODO: returns write check to queue later
            # DbWriteCheck(self.stdout),
            MigrateCheck(self.stdout),
            CachesAvailableCheck(self.stdout),
            CeleryBrokerAvailableCheck(self.stdout),
        ))

    def handle(self, *args: Tuple[Any, ...], **options: Dict[str, Any]) -> None:  # noqa: ARG002
        """Command implementation."""
        self._attempts = cast(int, options.get("n", self._attempts))
        self._timeout = cast(int, options.get("t", self._timeout))

        self.register_checks()
        while self._checks_queue and self._attempts > 0:
            checker = self._checks_queue.popleft()
            if not checker.check():
                self._checks_queue.appendleft(checker)
                self._attempts -= 1
                if self._attempts > 0:
                    time.sleep(self._timeout)
                    continue
                sys.exit(1)



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class DjangoheadsConfig(AppConfig):
    """DjangoHeads app config."""

    name = "djangoheads"
    verbose_name = "DjangoHeads Core Library"

    def ready(self) -> None:
        """Initializes the app."""
        self.init_sentry_sdk()

    def init_sentry_sdk(self) -> None:
        """Initializes sentry_sdk if SENTRY_DSN is set."""
        sentry_dsn = getattr(settings, "SENTRY_DSN", None)
        if not sentry_dsn:
            return

        try:
            import sentry_sdk  # noqa: PLC0415
            from sentry_sdk.integrations.django import DjangoIntegration  # noqa: PLC0415
        except ImportError:
            raise ImproperlyConfigured("sentry_sdk is not installed but SENTRY_DSN is set")

        sentry_sdk.init(
            dsn=sentry_dsn,
            integrations=[
                DjangoIntegration(),
            ],
            traces_sample_rate=getattr(settings, "SENTRY_TRACES_SAMPLE_RATE", 0.1),
            send_default_pii=settings.DEBUG,
            release=getattr(settings, "SENTRY_RELEASE", "service@release-undefined"),
        )



from django.conf import settings
from django.core.cache import InvalidCacheBackendError, caches
from django.core.exceptions import ImproperlyConfigured

from .check__base import CheckBase


class CachesAvailableCheck(CheckBase):
    """Check caches availability."""

    message_template = "CACHE IS AVAILABLE [{}]"

    def check(self) -> bool:
        """Check method implementation."""
        try:
            for cache_name in settings.CACHES:
                self.message = self.message_template.format(cache_name)
                cache = caches[cache_name]
                test_key = f"__test_{self._get_random_hexstr()}__"
                test_value = "1"
                cache.set(test_key, test_value, timeout=60)
                if cache.get(test_key) != test_value:
                    self._print_check(self.message, False)
                    break
            else:
                self._print_check(self.message, True)
                return True
        except InvalidCacheBackendError as exc:
            if isinstance(exc, ImproperlyConfigured):
                self._print_check("CACHES ARE NOT CONFIGURED", None)
                return True
            self._print_check(self.message, False, exception=exc)
        except Exception as exc:
            self._print_check(self.message, exception=exc)
        return False
